# -----------------------------------------------------------------------------
# Endroid - XMPP Bot - Pounce user presence plugin
# Copyright 2014, Ensoft Ltd.
# Created by Patrick Stevens
# -----------------------------------------------------------------------------

"""Sample plugin to demonstrate use of UserManagement.register_presence_callback"""

from endroid.plugins.command import CommandPlugin, command

class Pounce(CommandPlugin):
    name = "pounce"
    help = ("Pounces on a user by sending them a message immediately when they"
            " come online. \n"
            "Use 'pounce message <user> <message>', \n"
            "or 'stalk <user>' (to be notified when a user goes offline), \n"
            "or 'pounce <user>' to be told when the user comes online.")

    
    @command(helphint='{user}')
    def pounce(self, msg, args):
        subject = args.strip()  # strip spaces from the outside if necessary
        if args[:8] == 'message ':
            # we don't want to handle this, as it's a 'pounce message' call
            msg.unhandled()
            return
        if self.rosters.is_online(subject):
            msg.reply(subject + ' is already online.')
            return

        message = subject + ' came online.'
        cb = lambda: self.messages.send_chat(msg.sender, message)

        self.rosters.register_presence_callback(user=subject, callback=cb,
                                               available=True)
        msg.reply("I'll let you know when {} comes online.".format(subject))

    @command(helphint="{user} {message}")
    def pounce_message(self, msg, args):
        args_split = args.split(' ')
        tojid = args_split[0]

        if args_split[1:]:
            message = msg.sender + ' pounced! ' + ' '.join(args_split[1:])
        else:
            msg.reply('You need to specify both a user and a message for me to '
                      'deliver when I pounce.')
            return

        if self.rosters.is_online(tojid):
            self.messages.send_chat(tojid, message, msg.sender)
            msg.reply("The user was already online, so I just told them direct.")
            return

        cb = lambda: self.messages.send_chat(tojid, message, msg.sender)

        self.rosters.register_presence_callback(user=tojid, callback=cb,
                                                       available=True)

        msg.reply("Lying in wait for {} with message '{}'".format(tojid, message))


    @command(helphint="{user}")
    def stalk(self, msg, args):
        cb = lambda: self.messages.send_chat(msg.sender,
                                             'User {} is offline'.format(args))
        self.rosters.register_presence_callback(user=args, callback=cb,
                                               unavailable=True)

        msg.reply("You will be notified when user {} goes offline.".format(args))
